import React from 'react'
import { mount } from 'enzyme'
import toJson from 'enzyme-to-json'
import { waitForComponentToPaint } from '../../../tests/__mocks__/testUtils'
import { DateTime } from 'luxon'
import ElecHalfHourMonthlyAnalysis from './ElecHalfHourMonthlyAnalysis'
import { IconButton } from '@material-ui/core'
import {
  mockDataLoadEnedisAnalysis,
  mockEnedisMonthlyAnalysisArray,
} from '../../../tests/__mocks__/enedisMonthlyAnalysisData.mock'
import { PerformanceIndicator } from 'models'
import { allLastFluidPrices } from '../../../tests/__mocks__/fluidPrice.mock'

jest.mock('cozy-ui/transpiled/react/I18n', () => {
  return {
    useI18n: jest.fn(() => {
      return {
        t: (str: string) => str,
      }
    }),
  }
})
jest.mock('components/Hooks/useExploration', () => {
  return () => ['', jest.fn()]
})

jest.mock(
  'components/Analysis/ElecHalfHourChart',
  () => 'mock-elechalfhourchart'
)
jest.mock('components/Analysis/ElecInfoModal', () => 'mock-elecinfomodal')

const mockCheckDoctypeEntries = jest.fn()
jest.mock('services/consumption.service', () => {
  return jest.fn(() => {
    return {
      checkDoctypeEntries: mockCheckDoctypeEntries,
    }
  })
})
const mockGetEnedisMonthlyAnalysisByDate = jest.fn()
const mockAggregateValuesToDataLoad = jest.fn()
jest.mock('services/enedisMonthlyAnalysisData.service', () => {
  return jest.fn(() => {
    return {
      getEnedisMonthlyAnalysisByDate: mockGetEnedisMonthlyAnalysisByDate,
      aggregateValuesToDataLoad: mockAggregateValuesToDataLoad,
    }
  })
})
const mockPerfIndicator: PerformanceIndicator = {
  value: 10,
  compareValue: null,
  percentageVariation: null,
  price: null,
}
const mockGetPrices = jest.fn()
jest.mock('services/fluidsPrices.service', () => {
  return jest.fn(() => {
    return {
      getPrices: mockGetPrices,
    }
  })
})

describe('ElecHalfHourMonthlyAnalysis component', () => {
  beforeEach(() => {
    mockCheckDoctypeEntries.mockClear()
    mockGetEnedisMonthlyAnalysisByDate.mockClear()
    mockAggregateValuesToDataLoad.mockClear()
  })

  it('should be rendered correctly when isHalfHourActivated is false', async () => {
    mockCheckDoctypeEntries.mockResolvedValueOnce(false)
    mockGetPrices.mockResolvedValue(allLastFluidPrices[0])
    const wrapper = mount(
      <ElecHalfHourMonthlyAnalysis
        analysisDate={DateTime.fromISO('2021-07-01T00:00:00.000Z', {
          zone: 'utc',
        })}
        perfIndicator={mockPerfIndicator}
      />
    )
    await waitForComponentToPaint(wrapper)
    expect(toJson(wrapper)).toMatchSnapshot()
  })

  it('should be rendered correctly when isHalfHourActivated is true', async () => {
    mockCheckDoctypeEntries.mockResolvedValue(true)
    mockGetEnedisMonthlyAnalysisByDate.mockResolvedValueOnce(
      mockEnedisMonthlyAnalysisArray
    )
    mockAggregateValuesToDataLoad.mockResolvedValueOnce(
      mockDataLoadEnedisAnalysis
    )
    mockGetPrices.mockResolvedValue(allLastFluidPrices[0])

    const wrapper = mount(
      <ElecHalfHourMonthlyAnalysis
        analysisDate={DateTime.fromISO('2021-07-01T00:00:00.000Z', {
          zone: 'utc',
        })}
        perfIndicator={mockPerfIndicator}
      />
    )
    await waitForComponentToPaint(wrapper)
    expect(toJson(wrapper)).toMatchSnapshot()
  })

  it('should change from weekend to week', async () => {
    mockCheckDoctypeEntries.mockResolvedValue(true)
    mockGetEnedisMonthlyAnalysisByDate.mockResolvedValueOnce(
      mockEnedisMonthlyAnalysisArray
    )
    mockAggregateValuesToDataLoad.mockResolvedValueOnce(
      mockDataLoadEnedisAnalysis
    )
    mockGetPrices.mockResolvedValue(allLastFluidPrices[0])
    const wrapper = mount(
      <ElecHalfHourMonthlyAnalysis
        analysisDate={DateTime.fromISO('2021-07-01T00:00:00.000Z', {
          zone: 'utc',
        })}
        perfIndicator={mockPerfIndicator}
      />
    )
    await waitForComponentToPaint(wrapper)
    wrapper.find(IconButton).first().simulate('click')
    await waitForComponentToPaint(wrapper)
    expect(wrapper.find('.week').exists()).toBeTruthy()
  })

  it('should call the ElecInfoModal with open = true when click on the button', async () => {
    mockCheckDoctypeEntries.mockResolvedValue(true)
    mockGetEnedisMonthlyAnalysisByDate.mockResolvedValueOnce(
      mockEnedisMonthlyAnalysisArray
    )
    mockAggregateValuesToDataLoad.mockReturnValueOnce(
      mockDataLoadEnedisAnalysis
    )
    mockGetPrices.mockResolvedValue(allLastFluidPrices[0])

    const wrapper = mount(
      <ElecHalfHourMonthlyAnalysis
        analysisDate={DateTime.fromISO('2021-07-01T00:00:00.000Z', {
          zone: 'utc',
        })}
        perfIndicator={mockPerfIndicator}
      />
    )
    await waitForComponentToPaint(wrapper)
    wrapper.find('.showmodal').first().simulate('click')
    await waitForComponentToPaint(wrapper)
    expect(wrapper.find('mock-elecinfomodal').prop('open')?.valueOf()).toBe(
      true
    )
  })
})
