import { Button } from '@material-ui/core'
import { ERROR_EVENT } from 'cozy-harvest-lib/dist/models/flowEvents'
import { FluidType } from 'enum/fluid.enum'
import { KonnectorError } from 'enum/konnectorError.enum'
import { mount } from 'enzyme'
import toJson from 'enzyme-to-json'
import React from 'react'
import { Provider } from 'react-redux'
import { accountsData } from '../../../tests/__mocks__/accountsData.mock'
import {
  createMockStore,
  mockInitialEcolyoState,
} from '../../../tests/__mocks__/store'
import { waitForComponentToPaint } from '../../../tests/__mocks__/testUtils'
import KonnectorModalFooter from './KonnectorModalFooter'

jest.mock('cozy-ui/transpiled/react/I18n', () => {
  return {
    useI18n: jest.fn(() => {
      return {
        t: (str: string) => str,
      }
    }),
  }
})
const mockedNavigate = jest.fn()
jest.mock('react-router-dom', () => ({
  ...jest.requireActual('react-router-dom'),
  useNavigate: () => mockedNavigate,
}))
const mockClose = jest.fn()
const mockDelete = jest.fn()
describe('KonnectorModalFooter component', () => {
  let store: MockStoreEnhanced<
    {
      ecolyo: {
        global: GlobalState
      }
    },
    {}
  >
  beforeEach(() => {
    store = createMockStore(mockInitialEcolyoState)
    mockDelete.mockClear()
    mockClose.mockClear()
    mockedNavigate.mockClear()
  })

  it('should be rendered correctly', () => {
    const wrapper = mount(
      <Provider store={store}>
        <KonnectorModalFooter
          state={null}
          error={null}
          fluidType={FluidType.ELECTRICITY}
          handleCloseClick={mockClose}
          handleAccountDeletion={mockDelete}
          account={null}
          isUpdating={false}
        />
      </Provider>
    )
    expect(toJson(wrapper)).toMatchSnapshot()
  })

  it('should render "understood" for enedis sge when postal code not supported (USER_ACTION_NEEDED)', () => {
    const wrapper = mount(
      <Provider store={store}>
        <KonnectorModalFooter
          state={ERROR_EVENT}
          error={KonnectorError.USER_ACTION_NEEDED}
          fluidType={FluidType.ELECTRICITY}
          handleCloseClick={mockClose}
          handleAccountDeletion={mockDelete}
          account={null}
          isUpdating={false}
        />
      </Provider>
    )
    expect(wrapper.find(Button).text()).toBe(
      'konnector_modal.button_understood'
    )
  })

  it('should render "come back later" for enedis sge when address / name / pdl are not matching (LOGIN_FAILED)', () => {
    const wrapper = mount(
      <Provider store={store}>
        <KonnectorModalFooter
          state={ERROR_EVENT}
          error={KonnectorError.LOGIN_FAILED}
          fluidType={FluidType.ELECTRICITY}
          handleCloseClick={mockClose}
          handleAccountDeletion={mockDelete}
          account={null}
          isUpdating={false}
        />
      </Provider>
    )
    expect(wrapper.find(Button).text()).toBe(
      'konnector_modal.button_come_back_later'
    )
  })

  it('should render "later" and "Verify infos" for enedis sge encountering an error on contract (TERMS_VERSION_MISMATCH)', () => {
    const wrapper = mount(
      <Provider store={store}>
        <KonnectorModalFooter
          state={ERROR_EVENT}
          error={KonnectorError.TERMS_VERSION_MISMATCH}
          fluidType={FluidType.ELECTRICITY}
          handleCloseClick={mockClose}
          handleAccountDeletion={mockDelete}
          account={null}
          isUpdating={false}
        />
      </Provider>
    )
    expect(wrapper.find(Button).at(0).text()).toBe('Plus tard')
    expect(wrapper.find(Button).at(1).text()).toBe('Vérifier les infos')
  })

  it('should close the modal', () => {
    const wrapper = mount(
      <Provider store={store}>
        <KonnectorModalFooter
          state={ERROR_EVENT}
          error={KonnectorError.LOGIN_FAILED}
          fluidType={FluidType.ELECTRICITY}
          handleCloseClick={mockClose}
          handleAccountDeletion={mockDelete}
          account={null}
          isUpdating={false}
        />
      </Provider>
    )
    wrapper.find(Button).first().simulate('click')
    expect(mockClose).toHaveBeenCalled()
  })
  it('should go back to login', async () => {
    const wrapper = mount(
      <Provider store={store}>
        <KonnectorModalFooter
          state={ERROR_EVENT}
          error={KonnectorError.TERMS_VERSION_MISMATCH}
          fluidType={FluidType.ELECTRICITY}
          handleCloseClick={mockClose}
          handleAccountDeletion={mockDelete}
          account={accountsData[0]}
          isUpdating={true}
        />
      </Provider>
    )
    wrapper.find(Button).at(1).simulate('click')
    await waitForComponentToPaint(wrapper)
    expect(mockDelete).toHaveBeenCalled()
    expect(mockedNavigate).toHaveBeenCalledTimes(1)
  })
  it('should reset sge account and go back to login', async () => {
    const wrapper = mount(
      <Provider store={store}>
        <KonnectorModalFooter
          state={ERROR_EVENT}
          error={KonnectorError.TERMS_VERSION_MISMATCH}
          fluidType={FluidType.ELECTRICITY}
          handleCloseClick={mockClose}
          handleAccountDeletion={mockDelete}
          account={accountsData[0]}
          isUpdating={true}
        />
      </Provider>
    )
    wrapper.find(Button).at(1).simulate('click')
    await waitForComponentToPaint(wrapper)
    expect(mockDelete).toHaveBeenCalled()
    expect(mockedNavigate).toHaveBeenCalledTimes(1)
  })
  it('should not reset account if no account', () => {
    const wrapper = mount(
      <Provider store={store}>
        <KonnectorModalFooter
          state={ERROR_EVENT}
          error={KonnectorError.TERMS_VERSION_MISMATCH}
          fluidType={FluidType.ELECTRICITY}
          handleCloseClick={mockClose}
          handleAccountDeletion={mockDelete}
          account={null}
          isUpdating={false}
        />
      </Provider>
    )
    wrapper.find(Button).at(1).simulate('click')
    expect(mockedNavigate).toHaveBeenCalledTimes(1)
    expect(mockDelete).toHaveBeenCalledTimes(0)
  })
})
