import { QueryResult } from 'cozy-client'
import { ENEDIS_MINUTE_DOCTYPE } from 'doctypes'
import { DataloadState } from 'enum/dataload.enum'
import { FluidType } from 'enum/fluid.enum'
import { TimeStep } from 'enum/timeStep.enum'
import { DateTime } from 'luxon'
import {
  Datachart,
  Dataload,
  DataloadEntity,
  FluidPrice,
  FluidStatus,
  TimePeriod,
} from 'models'
import mockClient from '../../tests/__mocks__/client'
import { fluidPrices } from '../../tests/__mocks__/fluidPrice.mock'
import { fluidStatusConnectedData } from '../../tests/__mocks__/fluidStatusData.mock'
import { loadDayData } from '../../tests/__mocks__/loadDayData.mock'
import ConsumptionDataManager from './consumption.service'

const mockFetchFluidData = jest.fn()
const mockFetchFluidMaxData = jest.fn()
const mockGetFirstDateData = jest.fn()
const mockGetLastDateData = jest.fn()
const mockGetEntries = jest.fn()
jest.mock('./queryRunner.service', () => {
  return jest.fn(() => {
    return {
      fetchFluidData: mockFetchFluidData,
      fetchFluidMaxData: mockFetchFluidMaxData,
      getFirstDateData: mockGetFirstDateData,
      getLastDateData: mockGetLastDateData,
      getEntries: mockGetEntries,
    }
  })
})

describe('Consumption service', () => {
  const consumptionDataManager = new ConsumptionDataManager(mockClient)
  const fluidStatus: FluidStatus[] = fluidStatusConnectedData
  const mockTimePeriod: TimePeriod = {
    startDate: DateTime.fromISO('2020-08-01T00:00:00.000Z'),
    endDate: DateTime.fromISO('2020-08-03T23:59:59.999Z'),
  }
  const mockTimePeriodComparison: TimePeriod = {
    startDate: DateTime.fromISO('2020-07-01T00:00:00.000Z'),
    endDate: DateTime.fromISO('2020-07-03T23:59:59.999Z'),
  }
  const mockFetchDataActual: Dataload[] = [
    {
      date: DateTime.fromISO('2020-08-01T00:00:00.000Z'),
      value: 291.9,
      state: DataloadState.VALID,
      valueDetail: null,
    },
    {
      date: DateTime.fromISO('2020-08-02T00:00:00.000Z'),
      value: 260.15,
      state: DataloadState.VALID,
      valueDetail: null,
    },
  ]
  const mockFetchDataComparison: Dataload[] = [
    {
      date: DateTime.fromISO('2020-07-01T00:00:00.000Z'),
      value: 228.23,
      state: DataloadState.VALID,
      valueDetail: null,
    },
    {
      date: DateTime.fromISO('2020-07-02T00:00:00.000Z'),
      value: 238.71,
      state: DataloadState.VALID,
      valueDetail: null,
    },
  ]

  describe('getGraphData method', () => {
    beforeEach(() => {
      mockFetchFluidData.mockClear()
    })
    it('should return null', async () => {
      const result = await consumptionDataManager.getGraphData(
        mockTimePeriod,
        TimeStep.DAY,
        [],
        [],
        mockTimePeriodComparison,
        false
      )
      expect(result).toBeNull()
    })
    it('should return a mapped data for one fluid', async () => {
      const fluidTypes: FluidType[] = [FluidType.ELECTRICITY]
      mockFetchFluidData.mockResolvedValueOnce(mockFetchDataActual)
      mockFetchFluidData.mockResolvedValueOnce(mockFetchDataComparison)
      const mockResult: Datachart = {
        actualData: [
          {
            date: DateTime.fromISO('2020-08-01T00:00:00.000Z'),
            value: 291.9,
            state: DataloadState.VALID,
            valueDetail: null,
          },
          {
            date: DateTime.fromISO('2020-08-02T00:00:00.000Z'),
            value: 260.15,
            state: DataloadState.VALID,
            valueDetail: null,
          },
          {
            date: DateTime.fromISO('2020-08-03T00:00:00.000Z'),
            value: -1,
            state: DataloadState.HOLE,
            valueDetail: null,
          },
        ],
        comparisonData: [
          {
            date: DateTime.fromISO('2020-07-01T00:00:00.000Z'),
            value: 228.23,
            state: DataloadState.VALID,
            valueDetail: null,
          },
          {
            date: DateTime.fromISO('2020-07-02T00:00:00.000Z'),
            value: 238.71,
            state: DataloadState.VALID,
            valueDetail: null,
          },
          {
            date: DateTime.fromISO('2020-07-03T00:00:00.000Z'),
            value: -1,
            state: DataloadState.HOLE,
            valueDetail: null,
          },
        ],
      }
      const result = await consumptionDataManager.getGraphData(
        mockTimePeriod,
        TimeStep.DAY,
        fluidTypes,
        fluidStatus,
        mockTimePeriodComparison,
        false
      )
      expect(result).toEqual(mockResult)
    })

    it('should return a mapped data for multiple fluid', async () => {
      const fluidTypes: FluidType[] = [
        FluidType.ELECTRICITY,
        FluidType.WATER,
        FluidType.GAS,
      ]

      for (const fluidType of fluidTypes) {
        mockFetchFluidData.mockResolvedValueOnce(mockFetchDataActual)
        mockFetchFluidData.mockResolvedValueOnce(mockFetchDataComparison)
      }

      const mockResult: Datachart = {
        actualData: [
          {
            date: DateTime.fromISO('2020-08-01T00:00:00.000Z'),
            value: 84.44375099999999,
            state: DataloadState.AGGREGATED_VALID,
            valueDetail: [
              { value: 50.79059999999999, state: DataloadState.VALID },
              { value: 0.931161, state: DataloadState.VALID },
              { value: 32.72199, state: DataloadState.VALID },
            ],
          },
          {
            date: DateTime.fromISO('2020-08-02T00:00:00.000Z'),
            value: 75.2587935,
            state: DataloadState.AGGREGATED_VALID,
            valueDetail: [
              { value: 45.266099999999994, state: DataloadState.VALID },
              { value: 0.8298785, state: DataloadState.VALID },
              { value: 29.162815, state: DataloadState.VALID },
            ],
          },
          {
            date: DateTime.fromISO('2020-08-03T00:00:00.000Z'),
            value: -1,
            state: DataloadState.AGGREGATED_HOLE_OR_MISSING,
            valueDetail: null,
          },
        ],
        comparisonData: [
          {
            date: DateTime.fromISO('2020-07-01T00:00:00.000Z'),
            value: 66.0246567,
            state: DataloadState.AGGREGATED_VALID,
            valueDetail: [
              { value: 39.712019999999995, state: DataloadState.VALID },
              { value: 0.7280537, state: DataloadState.VALID },
              { value: 25.584583, state: DataloadState.VALID },
            ],
          },
          {
            date: DateTime.fromISO('2020-07-02T00:00:00.000Z'),
            value: 69.05641589999999,
            state: DataloadState.AGGREGATED_VALID,
            valueDetail: [
              { value: 41.53554, state: DataloadState.VALID },
              { value: 0.7614849, state: DataloadState.VALID },
              { value: 26.759391, state: DataloadState.VALID },
            ],
          },
          {
            date: DateTime.fromISO('2020-07-03T00:00:00.000Z'),
            value: -1,
            state: DataloadState.AGGREGATED_HOLE_OR_MISSING,
            valueDetail: null,
          },
        ],
      }
      const result = await consumptionDataManager.getGraphData(
        mockTimePeriod,
        TimeStep.DAY,
        fluidTypes,
        fluidStatus,
        mockTimePeriodComparison,
        true
      )
      expect(result).toEqual(mockResult)
    })
    it('should return a mapped data for multi fluid without comparison date', async () => {
      const fluidTypes: FluidType[] = [
        FluidType.ELECTRICITY,
        FluidType.WATER,
        FluidType.GAS,
      ]
      const mockResult: Datachart = {
        actualData: [
          {
            date: DateTime.fromISO('2020-08-01T00:00:00.000Z'),
            value: 84.44375099999999,
            state: DataloadState.AGGREGATED_VALID,
            valueDetail: [
              { value: 50.79059999999999, state: DataloadState.VALID },
              { value: 0.931161, state: DataloadState.VALID },
              { value: 32.72199, state: DataloadState.VALID },
            ],
          },
          {
            date: DateTime.fromISO('2020-08-02T00:00:00.000Z'),
            value: 75.2587935,
            state: DataloadState.AGGREGATED_VALID,
            valueDetail: [
              { value: 45.266099999999994, state: DataloadState.VALID },
              { value: 0.8298785, state: DataloadState.VALID },
              { value: 29.162815, state: DataloadState.VALID },
            ],
          },
          {
            date: DateTime.fromISO('2020-08-03T00:00:00.000Z'),
            value: -1,
            state: DataloadState.AGGREGATED_HOLE_OR_MISSING,
            valueDetail: null,
          },
        ],
        comparisonData: [],
      }
      mockFetchFluidData.mockResolvedValue(mockFetchDataActual)
      const result = await consumptionDataManager.getGraphData(
        mockTimePeriod,
        TimeStep.DAY,
        fluidTypes,
        fluidStatus
      )
      expect(result).toEqual(mockResult)
    })
    it('should return null because of wrong parameters', async () => {
      const fluidTypes = [FluidType.WATER]
      const result = await consumptionDataManager.getGraphData(
        mockTimePeriod,
        TimeStep.HALF_AN_HOUR,
        fluidTypes,
        fluidStatus,
        mockTimePeriodComparison,
        true
      )
      expect(result).toBeNull()
    })
    it('should return null because of timePeriod and comparaison', async () => {
      const fluidTypes: FluidType[] = [FluidType.ELECTRICITY]
      const wrongTimePeriod = {
        startDate: DateTime.fromISO('2020-08-03T23:59:59.999Z'),
        endDate: DateTime.fromISO('2020-08-01T00:00:00.000Z'),
      }
      const result = await consumptionDataManager.getGraphData(
        wrongTimePeriod,
        TimeStep.DAY,
        fluidTypes,
        fluidStatus,
        mockTimePeriodComparison,
        true
      )
      expect(result).toBeNull()
    })
  })

  describe('getMaxLoad method', () => {
    it('should return the maxed value for a time period for the home', async () => {
      const fluidTypes: FluidType[] = [
        FluidType.ELECTRICITY,
        FluidType.WATER,
        FluidType.GAS,
      ]

      for (const fluidtype of fluidTypes) {
        mockFetchFluidData.mockResolvedValueOnce(mockFetchDataActual)
        mockFetchFluidData.mockResolvedValueOnce(mockFetchDataComparison)
      }
      const expectedResult = 84.44375099999999
      const result = await consumptionDataManager.getMaxLoad(
        mockTimePeriod,
        TimeStep.DAY,
        fluidTypes,
        mockTimePeriodComparison,
        true
      )
      expect(result).toEqual(expectedResult)
    })
    it('should return the maxed value for a time period', async () => {
      const fluidTypes: FluidType[] = [FluidType.ELECTRICITY]
      const expectedResult = 63.1254
      mockFetchFluidMaxData.mockResolvedValueOnce(expectedResult)
      const result = await consumptionDataManager.getMaxLoad(
        mockTimePeriod,
        TimeStep.DAY,
        fluidTypes,
        mockTimePeriodComparison,
        false
      )
      expect(result).toEqual(expectedResult)
    })
  })

  describe('getPerformanceIndicators method', () => {
    it('should return the performance indicator', async () => {
      const fluidTypes: FluidType[] = [
        FluidType.ELECTRICITY,
        FluidType.WATER,
        FluidType.GAS,
      ]
      mockFetchFluidData.mockResolvedValueOnce(mockFetchDataActual)
      mockFetchFluidData.mockResolvedValueOnce(mockFetchDataComparison)
      mockFetchFluidData.mockResolvedValueOnce(mockFetchDataActual)
      mockFetchFluidData.mockResolvedValueOnce(mockFetchDataComparison)
      //Incomplete Data to test all possibilities
      mockFetchFluidData.mockResolvedValueOnce([
        {
          date: DateTime.fromISO('2020-08-01T00:23:20.000Z'),
          value: 298.283,
        },
      ])
      mockFetchFluidData.mockResolvedValueOnce([
        {
          date: DateTime.fromISO('2020-08-01T03:10:00.000Z'),
          value: 398.283,
        },
      ])

      const mockTimePeriodComplete = {
        startDate: DateTime.fromISO('2020-08-01T00:00:00.000Z'),
        endDate: DateTime.fromISO('2020-08-02T23:59:59.999Z'),
      }
      const mockTimePeriodComparisonComplete = {
        startDate: DateTime.fromISO('2020-07-01T00:00:00.000Z'),
        endDate: DateTime.fromISO('2020-07-02T23:59:59.999Z'),
      }
      const expectedResult = [
        {
          compareValue: 466.94,
          percentageVariation: 0.18227181222426858,
          value: 552.05,
          price: null,
        },
        {
          compareValue: 466.94,
          percentageVariation: 0.18227181222426858,
          value: 552.05,
          price: null,
        },
        {
          compareValue: null,
          percentageVariation: null,
          value: null,
          price: null,
        },
      ]
      const result = await consumptionDataManager.getPerformanceIndicators(
        mockTimePeriodComplete,
        TimeStep.DAY,
        fluidTypes,
        mockTimePeriodComparisonComplete
      )
      expect(result).toEqual(expectedResult)
    })
  })

  describe('getExportableFluids method', () => {
    it('should return the array of fluidtypes that have entries', async () => {
      const fluidTypes: FluidType[] = [
        FluidType.ELECTRICITY,
        FluidType.WATER,
        FluidType.GAS,
      ]
      mockGetEntries.mockResolvedValueOnce({ data: [1] })
      mockGetEntries.mockResolvedValueOnce({ data: [] })
      mockGetEntries.mockResolvedValueOnce({ data: [1] })
      const result = await consumptionDataManager.getExportableFluids(
        fluidTypes,
        TimeStep.MONTH
      )
      expect(result).toStrictEqual([FluidType.ELECTRICITY, FluidType.GAS])
    })
  })

  describe('fetchAllFirstDateData method', () => {
    it('should return the latest date data of one fluid', async () => {
      const fluidTypes: FluidType[] = [FluidType.ELECTRICITY]
      const expectedResult = [DateTime.fromISO('2020-09-03T23:59:59.999Z')]
      mockGetFirstDateData.mockResolvedValueOnce(
        DateTime.fromISO('2020-09-03T23:59:59.999Z')
      )
      const result = await consumptionDataManager.fetchAllFirstDateData(
        fluidTypes
      )
      expect(result).toEqual(expectedResult)
    })
    it('should return the latest date data of All fluid', async () => {
      const fluidTypes: FluidType[] = [
        FluidType.ELECTRICITY,
        FluidType.WATER,
        FluidType.GAS,
      ]
      mockGetFirstDateData.mockResolvedValueOnce(
        DateTime.fromISO('2020-09-02T23:59:59.999Z')
      )
      mockGetFirstDateData.mockResolvedValueOnce(
        DateTime.fromISO('2020-09-03T23:59:59.999Z')
      )
      mockGetFirstDateData.mockResolvedValueOnce(
        DateTime.fromISO('2020-09-01T23:59:59.999Z')
      )
      const expectedResult = [
        DateTime.fromISO('2020-09-02T23:59:59.999Z'),
        DateTime.fromISO('2020-09-03T23:59:59.999Z'),
        DateTime.fromISO('2020-09-01T23:59:59.999Z'),
      ]
      const result = await consumptionDataManager.fetchAllFirstDateData(
        fluidTypes
      )
      expect(result).toEqual(expectedResult)
    })
  })

  describe('fetchAllLastDateData method', () => {
    it('should return the latest date data of one fluid', async () => {
      const fluidTypes: FluidType[] = [FluidType.ELECTRICITY]
      const expectedResult = [DateTime.fromISO('2020-09-03T23:59:59.999Z')]
      mockGetLastDateData.mockResolvedValueOnce(
        DateTime.fromISO('2020-09-03T23:59:59.999Z')
      )
      const result = await consumptionDataManager.fetchAllLastDateData(
        fluidTypes
      )
      expect(result).toEqual(expectedResult)
    })
    it('should return the latest date data of All fluid', async () => {
      const fluidTypes: FluidType[] = [
        FluidType.ELECTRICITY,
        FluidType.WATER,
        FluidType.GAS,
      ]
      mockGetLastDateData.mockResolvedValueOnce(
        DateTime.fromISO('2020-09-02T23:59:59.999Z')
      )
      mockGetLastDateData.mockResolvedValueOnce(
        DateTime.fromISO('2020-09-03T23:59:59.999Z')
      )
      mockGetLastDateData.mockResolvedValueOnce(
        DateTime.fromISO('2020-09-01T23:59:59.999Z')
      )
      const expectedResult = [
        DateTime.fromISO('2020-09-02T23:59:59.999Z'),
        DateTime.fromISO('2020-09-03T23:59:59.999Z'),
        DateTime.fromISO('2020-09-01T23:59:59.999Z'),
      ]
      const result = await consumptionDataManager.fetchAllLastDateData(
        fluidTypes
      )
      expect(result).toEqual(expectedResult)
    })
  })

  describe('checkDoctypeEntries method', () => {
    it('should return a boolean if doctype are correct', async () => {
      let fluidType: FluidType = FluidType.GAS
      mockGetEntries.mockResolvedValueOnce({ data: [1] })
      let result = await consumptionDataManager.checkDoctypeEntries(
        fluidType,
        TimeStep.DAY
      )
      expect(result).toBeTruthy()
      fluidType = 999
      mockGetEntries.mockResolvedValueOnce({ data: [] })
      result = await consumptionDataManager.checkDoctypeEntries(
        fluidType,
        TimeStep.DAY
      )
      expect(result).toBeFalsy()
    })
  })

  describe('getFirstDataDateFromDoctypeWithPrice', () => {
    it('should Get the first entry of a given data doctype', async () => {
      const data: QueryResult<FluidPrice[]> = {
        data: [fluidPrices[0]],
        bookmark: '',
        next: false,
        skip: 0,
      }

      mockClient.query.mockResolvedValueOnce(data)
      const result =
        await consumptionDataManager.getFirstDataDateFromDoctypeWithPrice(
          ENEDIS_MINUTE_DOCTYPE
        )
      expect(result).toEqual(data.data[0])
    })
  })

  describe('getLastHourData', () => {
    it('should get last hour data', async () => {
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: [loadDayData[0]],
        bookmark: '',
        next: false,
        skip: 0,
      }

      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await consumptionDataManager.getLastHourData(
        mockClient,
        12
      )
      expect(result).toEqual(mockQueryResult.data)
      expect(result.length).toEqual(1)
    })
  })

  describe('saveDoc & saveDocs', () => {
    it('should saveDoc', async () => {
      const mockQueryResult: QueryResult<DataloadEntity> = {
        data: loadDayData[0],
        bookmark: '',
        next: false,
        skip: 0,
      }

      mockClient.save.mockResolvedValueOnce(mockQueryResult)
      const result = await consumptionDataManager.saveDoc(loadDayData[0])
      expect(result).toEqual(mockQueryResult.data)
    })
    it('should saveDocs', async () => {
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadDayData,
        bookmark: '',
        next: false,
        skip: 0,
      }

      mockClient.saveAll.mockResolvedValueOnce(mockQueryResult)
      const result = await consumptionDataManager.saveDocs(loadDayData)
      expect(result).toEqual(mockQueryResult.data)
      expect(result.length).toEqual(4)
    })
  })
})
