import { QueryResult } from 'cozy-client'
import { DateTime } from 'luxon'
import { Profile } from 'models'
import mockClient from '../../tests/__mocks__/client'
import { profileData } from '../../tests/__mocks__/profileData.mock'
import ProfileService from './profile.service'

describe('UserProfile service', () => {
  const profileService = new ProfileService(mockClient)

  describe('getUserProfile', () => {
    it('should return the user profile', async () => {
      const mockQueryResult: QueryResult<Profile[]> = {
        data: [profileData],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await profileService.getProfile()
      expect(result).toEqual(profileData)
    })

    it('should return the user profile from string monthlyAnalysisDate', async () => {
      const userProfile = {
        ...profileData,
        monthlyAnalysisDate: '2020-11-09T11:27:30.073Z',
      }
      // eslint-disable-next-line @typescript-eslint/no-explicit-any
      const mockQueryResult: QueryResult<any[]> = {
        data: [userProfile],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const resultUserProfile = {
        ...profileData,
        monthlyAnalysisDate: DateTime.fromISO('2020-11-09T11:27:30.073Z', {
          zone: 'utc',
        }),
      }
      const result = await profileService.getProfile()
      expect(result).toEqual(resultUserProfile)
    })

    it('should return null if no user profile found', async () => {
      const mockQueryResult: QueryResult<Profile[]> = {
        data: [],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await profileService.getProfile()
      expect(result).toBeNull()
    })
  })

  describe('updateUserProfile', () => {
    it('should return an updated user profile', async () => {
      const userProfile = {
        ...profileData,
        monthlyAnalysisDate: '2020-11-03T00:00:00.000Z',
        customPopupDate: '2020-11-03T00:00:00.000Z',
      }
      // eslint-disable-next-line @typescript-eslint/no-explicit-any
      const mockQueryResult: QueryResult<any[]> = {
        data: [userProfile],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const updateUserProfile = {
        ...userProfile,
        haveSeenLastAnalysis: false,
      }
      // eslint-disable-next-line @typescript-eslint/no-explicit-any
      const mockUpdatedQueryResult: QueryResult<any> = {
        data: updateUserProfile,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.save.mockResolvedValueOnce(mockUpdatedQueryResult)
      const resultUserProfile = {
        ...profileData,
        monthlyAnalysisDate: DateTime.fromISO('2020-11-03T00:00:00.000Z', {
          zone: 'utc',
        }),
        customPopupDate: DateTime.fromISO('2020-11-03T00:00:00.000Z', {
          zone: 'utc',
        }),
        haveSeenLastAnalysis: false,
      }
      const result = await profileService.updateProfile({
        haveSeenLastAnalysis: false,
      })

      expect(result).toEqual(resultUserProfile)
    })

    it('should return null if no user profile found', async () => {
      const mockQueryResult: QueryResult<Profile[]> = {
        data: [profileData],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const mockUpdatedQueryResult: QueryResult<null> = {
        data: null,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.save.mockResolvedValueOnce(mockUpdatedQueryResult)
      const result = await profileService.updateProfile({
        haveSeenLastAnalysis: false,
      })
      expect(result).toBeNull()
    })
  })
})
