import {
  Floor,
  HotWaterEquipment,
  HousingType,
  IndividualInsulationWork,
  IndividualOrCollective,
  ProfileTypeStepForm,
  ThreeChoicesAnswer,
  WarmingType,
} from 'enum/profileType.enum'
import { DateTime } from 'luxon'
import { ProfileType } from 'models/profileType.model'
import mockClient from '../../tests/__mocks__/client'
import {
  mockCorrectedConsumption,
  mockEstimatedConsumption,
  mockMonthColdWaterConsumption,
  mockMonthConsumption,
  mockMonthConsumption1,
  mockMonthConsumption2,
  mockMonthCookingConsumption,
  mockMonthEcsConsumption1Solar,
  mockMonthEcsConsumptionOther,
  mockMonthEcsConsumptionThermo,
  mockMonthElectricSpecificConsumption,
  mockMonthlyForecastJanuaryTest1WithFullArrays,
  mockMonthlyForecastJanuaryTestProfile1,
  mockMonthlyForecastJanuaryTestProfile2,
  mockMonthlyForecastJanuaryTestProfile3,
  mockProfileType,
  mockProfileType1,
  mockProfileType2,
  mockProfileTypeAnswers,
  mockTestProfile1,
  mockTestProfile2,
  mockTestProfile3,
  mockWaterRawNeeds,
  mockWaterSpreadNeeds,
} from '../../tests/__mocks__/profileType.mock'
import ProfileTypeService from './profileType.service'
import ProfileTypeFormService from './profileTypeForm.service'

const wrongNumber = 99999
describe('ProfileType service', () => {
  const profileTypeService = new ProfileTypeService(
    mockProfileType,
    mockClient,
    DateTime.now().year
  )

  const profileTypeFormService = new ProfileTypeFormService(mockProfileType)

  describe('calculateWarmingEstimatedConsumption', () => {
    it('should calculate the Warming Estimated Consumption', () => {
      const estimatedConsumption =
        profileTypeService.calculateWarmingEstimatedConsumption()
      expect(estimatedConsumption).toEqual(mockEstimatedConsumption)
    })

    it('should calculate the Warming Corrected Consumption', () => {
      const correctedConsumption =
        profileTypeService.calculateWarmingCorrectedConsumption(
          mockEstimatedConsumption
        )
      expect(correctedConsumption).toEqual(mockCorrectedConsumption)
    })
    it('should calculate the Warming Month Consumption', async () => {
      const monthConsumption =
        await profileTypeService.calculateWarmingMonthConsumption(
          mockCorrectedConsumption,
          3
        )
      expect(monthConsumption).toEqual(mockMonthConsumption)
    })
    it('should get the heating consumption', async () => {
      const monthConsumption = await profileTypeService.getMonthHeating(3)
      expect(monthConsumption).toEqual(mockMonthConsumption)
    })
  })
  describe('should get the heating consumption for a flat with collective heating', () => {
    const _profileTypeService = new ProfileTypeService(
      mockProfileType1,
      mockClient,
      DateTime.now().year
    )
    it('should get the heating consumption', async () => {
      const monthConsumption = await _profileTypeService.getMonthHeating(2)
      expect(monthConsumption).toEqual(mockMonthConsumption1)
    })
  })
  describe('should get the heating consumption for a house with individual heating, facilities, installation and individual work', () => {
    const _profileTypeService = new ProfileTypeService(
      mockProfileType2,
      mockClient,
      DateTime.now().year
    )
    it('should get the heating consumption', async () => {
      const monthConsumption = await _profileTypeService.getMonthHeating(1)
      expect(monthConsumption).toEqual(mockMonthConsumption2)
    })
  })
  describe('calculateMonthWaterRawNeeds', () => {
    it('should calculate the water raw needs consumption by month', () => {
      const waterRawNeeds = profileTypeService.calculateMonthWaterRawNeeds(
        mockProfileType,
        1
      )
      expect(waterRawNeeds).toEqual(mockWaterRawNeeds)
    })

    it('should calculate the water spread needs Consumption', () => {
      const waterSpreadNeeds = profileTypeService.calculateSpreadNeeds(
        mockProfileType,
        1
      )
      expect(waterSpreadNeeds).toEqual(mockWaterSpreadNeeds)
    })
    it('should calculate the total ecs consumption by month if profileType.hotWaterEquipment === "solar"', () => {
      const monthEcsConsumption = profileTypeService.calculateTotalConsumption(
        mockWaterSpreadNeeds,
        mockProfileType1,
        1
      )
      expect(monthEcsConsumption).toEqual(mockMonthEcsConsumption1Solar)
    })
    it('should calculate the total ecs consumption by month if profileType.hotWaterEquipment === "other"', () => {
      const mockProfileTypeWithOther = { ...mockProfileType }
      mockProfileTypeWithOther.hotWaterEquipment = HotWaterEquipment.OTHER
      const monthEcsConsumption = profileTypeService.calculateTotalConsumption(
        mockWaterSpreadNeeds,
        mockProfileTypeWithOther,
        1
      )
      expect(monthEcsConsumption).toEqual(mockMonthEcsConsumptionOther)
    })
    it('should get the ECS consumption if profileType.hotWaterEquipment === "thermodynamic"', () => {
      const mockProfileTypeWithOther = { ...mockProfileType }
      mockProfileTypeWithOther.hotWaterEquipment =
        HotWaterEquipment.THERMODYNAMIC
      const monthConsumption = profileTypeService.getMonthEcs(1)
      expect(monthConsumption).toEqual(mockMonthEcsConsumptionThermo)
    })

    describe('should get the month cooking consumption', () => {
      it('should get the month cooking consumption', () => {
        const _profileTypeService = new ProfileTypeService(
          mockProfileType,
          mockClient,
          DateTime.now().year
        )

        const monthCookingConsumption =
          _profileTypeService.getMonthCookingConsumption(1)
        expect(monthCookingConsumption).toEqual(mockMonthCookingConsumption)
      })
    })
    describe('should get the month electric specific consumption', () => {
      it('should get the electric specific consumption', () => {
        const _profileTypeService = new ProfileTypeService(
          mockProfileType,
          mockClient,
          DateTime.now().year
        )

        const monthElectricSpecificConsumption =
          _profileTypeService.getMonthElectricSpecificConsumption(1)
        expect(monthElectricSpecificConsumption).toEqual(
          mockMonthElectricSpecificConsumption
        )
      })
    })
    describe('should get the month cold water consumption', () => {
      it('should get the cold water consumption', () => {
        const _profileTypeService = new ProfileTypeService(
          mockProfileType,
          mockClient,
          DateTime.now().year
        )
        const monthColdWaterConsumption =
          _profileTypeService.getMonthColdWaterConsumption(1)
        expect(monthColdWaterConsumption).toEqual(mockMonthColdWaterConsumption)
      })
    })
  })
  describe('should get the monthly Forecast', () => {
    // For month of january
    const profileTypeService1 = new ProfileTypeService(
      mockTestProfile1,
      mockClient,
      DateTime.now().year
    )
    it('should get the monthly forecast for test profile 1', async () => {
      const monthlyForecast = await profileTypeService1.getMonthlyForecast(1)
      expect(monthlyForecast).toEqual(mockMonthlyForecastJanuaryTestProfile1)
    })
    const profileTypeService2 = new ProfileTypeService(
      mockTestProfile2,
      mockClient,
      DateTime.now().year
    )
    it('should get the monthly forecast for test profile 2', async () => {
      const monthlyForecast = await profileTypeService2.getMonthlyForecast(1)
      expect(monthlyForecast).toEqual(mockMonthlyForecastJanuaryTestProfile2)
    })
    const profileTypeService3 = new ProfileTypeService(
      mockTestProfile3,
      mockClient,
      DateTime.now().year
    )
    it('should get the monthly forecast for test profile 3', async () => {
      const monthlyForecast = await profileTypeService3.getMonthlyForecast(1)
      expect(monthlyForecast).toEqual(mockMonthlyForecastJanuaryTestProfile3)
    })
    it('should get the monthly forecast for test profile 1 with wall and roof and window insulation', async () => {
      const mockProfileWithThreeInsulation: ProfileType = {
        ...mockTestProfile1,
        individualInsulationWork: [
          IndividualInsulationWork.ROOF_INSULATION,
          IndividualInsulationWork.WALL_INSULATION,
          IndividualInsulationWork.WINDOW_REPLACEMENT,
        ],
        hasInstalledVentilation: ThreeChoicesAnswer.YES,
        hasReplacedHeater: ThreeChoicesAnswer.YES,
      }
      const _profileTypeService = new ProfileTypeService(
        mockProfileWithThreeInsulation,
        mockClient,
        DateTime.now().year
      )
      const monthlyForecast = await _profileTypeService.getMonthlyForecast(1)
      expect(monthlyForecast).toEqual(
        mockMonthlyForecastJanuaryTest1WithFullArrays
      )
    })

    it('should get the monthly forecast with null heating for a profile with fuel or wood heating', async () => {
      const profileTypeService2 = new ProfileTypeService(
        { ...mockTestProfile2, warmingFluid: WarmingType.FUEL },
        mockClient,
        DateTime.now().year
      )
      const monthlyForecast = await profileTypeService2.getMonthlyForecast(1)
      expect(
        monthlyForecast.fluidForecast[0].detailsMonthlyForecast
          .heatingConsumption
      ).toEqual(null)
      expect(
        monthlyForecast.fluidForecast[1].detailsMonthlyForecast
          .heatingConsumption
      ).toEqual(null)
      expect(
        monthlyForecast.fluidForecast[2].detailsMonthlyForecast
          .heatingConsumption
      ).toEqual(null)
    })
  })
  describe('getNextFormStep', () => {
    it('should get the next step in function of the current step', () => {
      let nextStep = profileTypeFormService.getNextFormStep(
        ProfileTypeStepForm.HOUSING_TYPE,
        false
      )
      expect(nextStep).toEqual(ProfileTypeStepForm.AREA)
      nextStep = profileTypeFormService.getNextFormStep(
        ProfileTypeStepForm.AREA,
        false
      )
      expect(nextStep).toEqual(ProfileTypeStepForm.OCCUPANTS_NUMBER)

      nextStep = profileTypeFormService.getNextFormStep(
        ProfileTypeStepForm.OCCUPANTS_NUMBER,
        false
      )
      expect(nextStep).toEqual(ProfileTypeStepForm.CONSTRUCTION_YEAR)
      nextStep = profileTypeFormService.getNextFormStep(
        ProfileTypeStepForm.CONSTRUCTION_YEAR,
        false
      )
      expect(nextStep).toEqual(ProfileTypeStepForm.OUTSIDE_FACING_WALLS)
      nextStep = profileTypeFormService.getNextFormStep(
        ProfileTypeStepForm.OUTSIDE_FACING_WALLS,
        false
      )
      expect(nextStep).toEqual(ProfileTypeStepForm.FLOOR)
      nextStep = profileTypeFormService.getNextFormStep(
        ProfileTypeStepForm.FLOOR,
        false
      )
      expect(nextStep).toEqual(ProfileTypeStepForm.HEATING)
      nextStep = profileTypeFormService.getNextFormStep(
        ProfileTypeStepForm.HEATING,
        false
      )
      expect(nextStep).toEqual(ProfileTypeStepForm.WARMING_FLUID)
      nextStep = profileTypeFormService.getNextFormStep(
        ProfileTypeStepForm.WARMING_FLUID,
        false
      )
      expect(nextStep).toEqual(ProfileTypeStepForm.VENTILATION)
      nextStep = profileTypeFormService.getNextFormStep(
        ProfileTypeStepForm.VENTILATION,
        false
      )
      expect(nextStep).toEqual(ProfileTypeStepForm.HEATER_REPLACEMENT)
      nextStep = profileTypeFormService.getNextFormStep(
        ProfileTypeStepForm.HEATER_REPLACEMENT,
        false
      )
      expect(nextStep).toEqual(ProfileTypeStepForm.COLD_WATER)
      nextStep = profileTypeFormService.getNextFormStep(
        ProfileTypeStepForm.COLD_WATER,
        false
      )
      expect(nextStep).toEqual(ProfileTypeStepForm.HOT_WATER)
      nextStep = profileTypeFormService.getNextFormStep(
        ProfileTypeStepForm.HOT_WATER,
        false
      )
      expect(nextStep).toEqual(ProfileTypeStepForm.HOT_WATER_FLUID)
      nextStep = profileTypeFormService.getNextFormStep(
        ProfileTypeStepForm.HOT_WATER_FLUID,
        false
      )
      expect(nextStep).toEqual(ProfileTypeStepForm.HOT_WATER_EQUIPMENT)
      nextStep = profileTypeFormService.getNextFormStep(
        ProfileTypeStepForm.HOT_WATER_EQUIPMENT,
        false
      )
      expect(nextStep).toEqual(ProfileTypeStepForm.COOKING_FLUID)
      nextStep = profileTypeFormService.getNextFormStep(
        ProfileTypeStepForm.COOKING_FLUID,
        false
      )
      expect(nextStep).toEqual(ProfileTypeStepForm.EQUIPMENTS)
      nextStep = profileTypeFormService.getNextFormStep(
        ProfileTypeStepForm.EQUIPMENTS,
        false
      )
      expect(nextStep).toEqual(ProfileTypeStepForm.UPDATE_DATE)
      nextStep = profileTypeFormService.getNextFormStep(
        ProfileTypeStepForm.UPDATE_DATE,
        false
      )
      expect(nextStep).toEqual(ProfileTypeStepForm.END)
      nextStep = profileTypeFormService.getNextFormStep(wrongNumber, false)
      expect(nextStep).toEqual(ProfileTypeStepForm.HOUSING_TYPE)
    })
  })
  describe('getPreviousFormStep', () => {
    it('should get the previus step in function of the current step', () => {
      let previousStep = profileTypeFormService.getPreviousFormStep(
        ProfileTypeStepForm.AREA
      )
      expect(previousStep).toEqual(ProfileTypeStepForm.HOUSING_TYPE)
      previousStep = profileTypeFormService.getPreviousFormStep(
        ProfileTypeStepForm.OCCUPANTS_NUMBER
      )
      expect(previousStep).toEqual(ProfileTypeStepForm.AREA)
      previousStep = profileTypeFormService.getPreviousFormStep(
        ProfileTypeStepForm.CONSTRUCTION_YEAR
      )
      expect(previousStep).toEqual(ProfileTypeStepForm.OCCUPANTS_NUMBER)
      previousStep = profileTypeFormService.getPreviousFormStep(
        ProfileTypeStepForm.OUTSIDE_FACING_WALLS
      )
      expect(previousStep).toEqual(ProfileTypeStepForm.CONSTRUCTION_YEAR)
      previousStep = profileTypeFormService.getPreviousFormStep(
        ProfileTypeStepForm.FLOOR
      )
      expect(previousStep).toEqual(ProfileTypeStepForm.OUTSIDE_FACING_WALLS)
      previousStep = profileTypeFormService.getPreviousFormStep(
        ProfileTypeStepForm.HEATING
      )
      expect(previousStep).toEqual(ProfileTypeStepForm.FLOOR)
      previousStep = profileTypeFormService.getPreviousFormStep(
        ProfileTypeStepForm.WARMING_FLUID
      )
      expect(previousStep).toEqual(ProfileTypeStepForm.HEATING)
      previousStep = profileTypeFormService.getPreviousFormStep(
        ProfileTypeStepForm.INDIVIDUAL_INSULATION_WORK
      )
      expect(previousStep).toEqual(ProfileTypeStepForm.WARMING_FLUID)
      previousStep = profileTypeFormService.getPreviousFormStep(
        ProfileTypeStepForm.VENTILATION
      )
      expect(previousStep).toEqual(ProfileTypeStepForm.WARMING_FLUID)
      previousStep = profileTypeFormService.getPreviousFormStep(
        ProfileTypeStepForm.HEATER_REPLACEMENT
      )
      expect(previousStep).toEqual(ProfileTypeStepForm.VENTILATION)
      previousStep = profileTypeFormService.getPreviousFormStep(
        ProfileTypeStepForm.HOT_WATER
      )
      expect(previousStep).toEqual(ProfileTypeStepForm.COLD_WATER)
      previousStep = profileTypeFormService.getPreviousFormStep(
        ProfileTypeStepForm.HOT_WATER_FLUID
      )
      expect(previousStep).toEqual(ProfileTypeStepForm.HOT_WATER)

      previousStep = profileTypeFormService.getPreviousFormStep(
        ProfileTypeStepForm.HOT_WATER_EQUIPMENT
      )
      expect(previousStep).toEqual(ProfileTypeStepForm.HOT_WATER_FLUID)
      previousStep = profileTypeFormService.getPreviousFormStep(
        ProfileTypeStepForm.COOKING_FLUID
      )
      expect(previousStep).toEqual(ProfileTypeStepForm.HOT_WATER_EQUIPMENT)
      previousStep = profileTypeFormService.getPreviousFormStep(wrongNumber)
      expect(previousStep).toEqual(ProfileTypeStepForm.HOUSING_TYPE)
    })
  })
  describe('getAnswerForStep', () => {
    it('should get the good answers in function of the current step', () => {
      let answers = ProfileTypeFormService.getAnswerForStep(
        ProfileTypeStepForm.HOUSING_TYPE
      )
      expect(answers).toEqual(mockProfileTypeAnswers[0])
      answers = ProfileTypeFormService.getAnswerForStep(
        ProfileTypeStepForm.AREA
      )
      expect(answers).toEqual(mockProfileTypeAnswers[1])
      answers = ProfileTypeFormService.getAnswerForStep(
        ProfileTypeStepForm.OCCUPANTS_NUMBER
      )
      expect(answers).toEqual(mockProfileTypeAnswers[2])
      answers = ProfileTypeFormService.getAnswerForStep(
        ProfileTypeStepForm.CONSTRUCTION_YEAR
      )
      expect(answers).toEqual(mockProfileTypeAnswers[3])
      answers = ProfileTypeFormService.getAnswerForStep(
        ProfileTypeStepForm.OUTSIDE_FACING_WALLS
      )
      expect(answers).toEqual(mockProfileTypeAnswers[4])
      answers = ProfileTypeFormService.getAnswerForStep(
        ProfileTypeStepForm.FLOOR
      )
      expect(answers).toEqual(mockProfileTypeAnswers[5])
      answers = ProfileTypeFormService.getAnswerForStep(
        ProfileTypeStepForm.HEATING
      )
      expect(answers).toEqual(mockProfileTypeAnswers[6])
      answers = ProfileTypeFormService.getAnswerForStep(
        ProfileTypeStepForm.WARMING_FLUID
      )
      expect(answers).toEqual(mockProfileTypeAnswers[7])
      answers = ProfileTypeFormService.getAnswerForStep(
        ProfileTypeStepForm.INDIVIDUAL_INSULATION_WORK
      )
      expect(answers).toEqual(mockProfileTypeAnswers[8])
      answers = ProfileTypeFormService.getAnswerForStep(
        ProfileTypeStepForm.VENTILATION
      )
      expect(answers).toEqual(mockProfileTypeAnswers[9])
      answers = ProfileTypeFormService.getAnswerForStep(
        ProfileTypeStepForm.HEATER_REPLACEMENT
      )
      expect(answers).toEqual(mockProfileTypeAnswers[10])
      answers = ProfileTypeFormService.getAnswerForStep(
        ProfileTypeStepForm.COLD_WATER
      )
      expect(answers).toEqual(mockProfileTypeAnswers[11])
      answers = ProfileTypeFormService.getAnswerForStep(
        ProfileTypeStepForm.HOT_WATER
      )
      expect(answers).toEqual(mockProfileTypeAnswers[12])
      answers = ProfileTypeFormService.getAnswerForStep(
        ProfileTypeStepForm.HOT_WATER_FLUID
      )
      expect(answers).toEqual(mockProfileTypeAnswers[13])
      answers = ProfileTypeFormService.getAnswerForStep(
        ProfileTypeStepForm.HOT_WATER_EQUIPMENT
      )
      expect(answers).toEqual(mockProfileTypeAnswers[14])
      answers = ProfileTypeFormService.getAnswerForStep(
        ProfileTypeStepForm.COOKING_FLUID
      )
      expect(answers).toEqual(mockProfileTypeAnswers[15])
      answers = ProfileTypeFormService.getAnswerForStep(wrongNumber)
      expect(answers).toEqual(mockProfileTypeAnswers[16])
    })
  })
  describe('checkConsistency', () => {
    it('should return consistent profile for individual house', () => {
      const mockProfile = {
        ...mockProfileType1,
        housingType: HousingType.INDIVIDUAL_HOUSE,
      }
      jest
        .spyOn(DateTime, 'local')
        .mockReturnValueOnce(
          DateTime.fromISO('2021-01-01T00:00:00.000Z', { zone: 'utc' })
        )

      const expectedResult = {
        ...mockProfile,
        floor: Floor.NOT_APPLICABLE,
        heating: IndividualOrCollective.INDIVIDUAL,
        coldWater: IndividualOrCollective.INDIVIDUAL,
        hotWater: IndividualOrCollective.INDIVIDUAL,
      }
      const result = ProfileTypeService.checkConsistency(mockProfile)
      expect(result).toEqual(expectedResult)
    })
    it('should return consistent profile for appartment', () => {
      const mockProfile = mockProfileType1
      jest
        .spyOn(DateTime, 'local')
        .mockReturnValueOnce(
          DateTime.fromISO('2021-01-01T00:00:00.000Z', { zone: 'utc' })
        )

      const expectedResult = {
        ...mockProfile,
        warmingFluid: null,
        individualInsulationWork: [],
        facilitiesInstallation: [],
        hotWaterFluid: null,
        hotWaterEquipment: HotWaterEquipment.OTHER,
      }
      const result = ProfileTypeService.checkConsistency(mockProfile)
      expect(result).toEqual(expectedResult)
    })
  })
  describe('getDjU', () => {
    it('should return default dju', async () => {
      const result = await profileTypeService.fetchDJU(2)
      expect(result).toEqual(363)
    })
  })
})
